# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

"""
CLI to manage ccsdb
"""

from __future__ import print_function

import argparse
import logging

import ccsdb

_log = logging.getLogger(__name__)


def setup_parser():
    '''
    Set the main arguments.
    '''
    parser = argparse.ArgumentParser(
        prog="ccsdb-cli",
        version=ccsdb.__version__
    )
    # General connection options
    parser.add_argument(
        '--debug', action='store_true',
        help="Outputs bunches of debugging info")

    subparsers = parser.add_subparsers(title='actions')

    # INIT_DB
    parser_init_db = subparsers.add_parser(
        'init_db', help='Initialize the database')
    parser_init_db.set_defaults(func=init_db)
    parser_init_db.add_argument(
        '--destructive', action="store_true", default=False,
        help="Drop all the tables before re-creating them")

    return parser


def init_db(args):
    """ Initialize the database. """
    from ccsdb.app import _db
    _log.info('Initializing the database')
    if _log.getEffectiveLevel() <= 10:
        _db.engine.echo = True

    if args.destructive:
        _log.info(' - Dropping all tables first')
        _db.drop_all()
    _db.create_all()
    _log.info('Done')


def main():
    ''' Main function '''
    # Set up parser for global args
    parser = setup_parser()
    # Parse the commandline
    try:
        args = parser.parse_args()
    except argparse.ArgumentTypeError as err:
        print("\nError: {0}".format(err))
        return 3

    logging.basicConfig()
    _log.setLevel(logging.INFO)
    if args.debug:
        _log.setLevel(logging.DEBUG)

    return_code = 0
    try:
        args.func(args)
    except KeyboardInterrupt:
        print("\nInterrupted by user.")
        return_code = 1
    except argparse.ArgumentError as err:
        _log.debug('ArgparseError')
        print('{0}'.format(err.message))
        return_code = 4
    except Exception as err:
        print('Error: {0}'.format(err))
        _log.exception("Generic error catched:")
        return_code = 2
    return return_code


if __name__ == '__main__':  # pragma: no cover
    main()
